package im.zego.realtimetranslationdemo;

import android.app.Application;
import android.text.TextUtils;
import android.view.TextureView;

import java.nio.ByteBuffer;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;

import im.zego.realtimetranslationdemo.rtcproxy.ExpressProxy;
import im.zego.realtimetranslation.ZegoRealtimeTranslation;

public class RTCEngineManager {
    private static final String TAG = "RTCEngineManager";
    public static final String STREAM_PREFIX = "zegotrans_";
    private static final String SUBTITLE_RECOGNIZING_TEXT_PREFIX = "zegoa2ti:";
    private static final String SUBTITLE_RECOGNIZED_TEXT_PREFIX = "zegoa2td:";
    private static RTCEngineManager instance = new RTCEngineManager();
    private ExpressProxy rtcEngine = new ExpressProxy(this);
    private String userID;
    private String userName;
    private String roomID;
    public String publishStreamID;
    private ArrayList<StreamInfo> streamInfoList = new ArrayList<>();
    private ArrayList<IRTCEventListener> rtcEventListenerList = new ArrayList<>();
    private boolean isInRoom = false;

    public static RTCEngineManager getInstance() {
        return instance;
    }

    private RTCEngineManager() {
    }

    public void initRtc(Application application) {
        rtcEngine.init(application);
    }

    public void unInitRtc(IExecuteCallback<String> callback) {
        rtcEngine.unInit(callback);
        streamInfoList.clear();
    }

    public void loginRoom(String roomID, String roomName,
                          String userID, String userName, IExecuteCallback<Object> callback) {
        this.userID = userID;
        this.userName = userName;
        this.roomID = roomID;
        rtcEngine.login(roomID, roomName, userID, userName, callback);
        isInRoom = true;
    }

    public void logoutRoom() {
        publishStreamID = "";
        if (isInRoom) {
            streamInfoList.clear();
            isInRoom = false;
            rtcEngine.logout();
        }
    }

    private void startPublishStream(String streamID) {
        rtcEngine.startPublishStream(streamID);
    }

    public void stopPublishStream() {
        rtcEngine.stopPublishStream();
    }

    public void startPlayingStream(String streamID, Object view) {
        rtcEngine.startPlayingStream(streamID, view);
    }

    public void stopPlayingStream(String streamID) {
        rtcEngine.stopPlayingStream(streamID);
    }

    public boolean isInRoom() {
        return isInRoom;
    }

    public void startPublishStream() {
        publishStreamID = STREAM_PREFIX + roomID + "_" + userID;
        startPublishStream(publishStreamID);
        boolean has = false;
        for (StreamInfo streamInfo: streamInfoList) {
            if (streamInfo.streamID.equals(publishStreamID)) {
                has = true;
                break;
            }
        }

        if (!has) {
            StreamInfo streamInfo = new StreamInfo(userID, userName, publishStreamID);
            streamInfoList.add(0, streamInfo);
        }
    }

    public void startPreview(TextureView textureView) {
        rtcEngine.startPreview(textureView);
    }

    public ArrayList<StreamInfo> getStreamInfoList() {
        return streamInfoList;
    }

    public void onStreamAdd(ArrayList<StreamInfo> streamInfoList) {
        this.streamInfoList.addAll(streamInfoList);
        for (IRTCEventListener listener: rtcEventListenerList) {
            listener.onStreamAdd(streamInfoList);
        }
    }

    public void onStreamRemove(ArrayList<StreamInfo> streamInfoList) {
        for (StreamInfo streamInfo: streamInfoList) {
            for (StreamInfo sInfo: this.streamInfoList) {
                if (sInfo.streamID.equals(streamInfo.streamID)) {
                    this.streamInfoList.remove(sInfo);
                    break;
                }
            }
        }
        for (IRTCEventListener listener: rtcEventListenerList) {
            listener.onStreamRemove(streamInfoList);
        }
    }

    private void sendSEIData(String data) {
        rtcEngine.sendSEIData(data);
    }

    public void sendRecognizingText(String text) {
        sendSEIData(SUBTITLE_RECOGNIZING_TEXT_PREFIX + text);
    }

    public void sendRecognizedText(String text) {
        sendSEIData(SUBTITLE_RECOGNIZED_TEXT_PREFIX + text);
    }

    public void onReceiveSEI(String streamID, byte[] data) {
        String str = new String(data, StandardCharsets.UTF_8);
        if (str.startsWith(SUBTITLE_RECOGNIZING_TEXT_PREFIX)) {
            String subtitle = str.substring(SUBTITLE_RECOGNIZING_TEXT_PREFIX.length());
            for (IRTCEventListener listener: rtcEventListenerList) {
                listener.onRecognizingText(subtitle);
            }
        } else if (str.startsWith(SUBTITLE_RECOGNIZED_TEXT_PREFIX)) {
            String subtitle = str.substring(SUBTITLE_RECOGNIZING_TEXT_PREFIX.length());
            for (IRTCEventListener listener: rtcEventListenerList) {
                listener.onRecognizedText(subtitle);
            }
        }
    }

    public void onCaptureAudioData(ByteBuffer byteBuffer, int dataLength, int value) {
//        Log.i(TAG, "onCapturedAudioData:" + dataLength + ", " + value);
        for (IRTCEventListener listener: rtcEventListenerList) {
            listener.onCaptureAudioData(byteBuffer, dataLength, value);
        }
    }

    public void onRoomStateChanged(RoomStateChangeReason reason) {
        if (reason == RoomStateChangeReason.Reconnected) {
            if (!TextUtils.isEmpty(publishStreamID)) {
                startPublishStream();
            }
        } else {
            logoutRoom();
            for (IRTCEventListener listener : rtcEventListenerList) {
                listener.onRoomStateChanged(reason);
            }
        }
    }

    public static abstract class IRTCEventListener {
        void onRecognizingText(String jsonStr) {};
        void onRecognizedText(String jsonStr) {};
        void onStreamAdd(ArrayList<StreamInfo> streamIDList) {};
        void onStreamRemove(ArrayList<StreamInfo> streamIDList) {};
        void onRoomStateChanged(RoomStateChangeReason reason) {};
        void onCaptureAudioData(ByteBuffer byteBuffer, int dataLength, int value) {};
    }

    public void addRTCEventListener(IRTCEventListener listener) {
        if (!rtcEventListenerList.contains(listener)) {
            rtcEventListenerList.add(listener);
        }
    }

    public void removeRTCEventListener(IRTCEventListener listener) {
        rtcEventListenerList.remove(listener);
    }

    public enum RoomStateChangeReason {
        KickOut,
        Disconnect,
        LoginFailed,
        Reconnected
    }

    public static class StreamInfo {
        public String userID;
        public String userName;
        public String streamID;
        public StreamInfo(String userID, String userName, String streamID) {
            this.userID = userID;
            this.userName = userName;
            this.streamID = streamID;
        }
    }
}
