package im.zego.realtimetranslationdemo.permissions.core

import android.content.Context
import android.content.pm.PackageManager
import android.os.Build
import android.util.Log
import androidx.annotation.Size
import androidx.appcompat.app.AppCompatActivity
import androidx.core.content.ContextCompat
import androidx.fragment.app.Fragment
import androidx.fragment.app.FragmentManager
import androidx.lifecycle.MutableLiveData
import im.zego.realtimetranslationdemo.permissions.support.PermissionSupport
import im.zego.realtimetranslationdemo.permissions.support.manufacturer.IPermissionSupport

/**
 * 基于livedata实现的一套动态权限请求库
 * 1.内部参考Glide，维护了一个不可见的Fragment，为了间接调用fragment的requestPermissions。
 * 2.因为国内各大厂家对权限控制api定义参差不齐，早在Android5.0就有了自己的动态权限系统，导致我们通过Android API拿不到是否授权的结果。
 *
 * 为了解决这个问题，在hasPermissions和requestPermissions做了厂商兼容。
 * 具体参考support包目录下的实现，对应权限的判断方式可以拓展，目前支持的不多
 *
 * !!注意如果要通过获取硬件资源去判断是否授权，此种方案可能会与sdk冲突导致判断异常
 */
internal class LivePermissions {

    companion object {
        const val TAG = "permissions"
    }

    constructor(activity: AppCompatActivity) {
        liveFragment = getInstance(activity.supportFragmentManager)
    }

    constructor(fragment: Fragment) {
        liveFragment = getInstance(fragment.childFragmentManager)
    }

    constructor()


    @Volatile
    private var liveFragment: LiveFragment? = null

    private fun getInstance(fragmentManager: FragmentManager) =
        liveFragment ?: synchronized(this) {
            liveFragment ?: if (fragmentManager.findFragmentByTag(TAG) == null) LiveFragment().run {
                fragmentManager.beginTransaction().add(this, TAG).commitNowAllowingStateLoss()
                this
            } else fragmentManager.findFragmentByTag(TAG) as LiveFragment
        }

    fun request(vararg permissions: String): MutableLiveData<PermissionResult> {
        return this.requestArray(permissions)
    }

    private fun requestArray(permissions: Array<out String>): MutableLiveData<PermissionResult> {
        liveFragment!!.requestPermissions(permissions)
        return liveFragment!!.liveData
    }

    fun permissionPermanentlyDenied(deniedPermission: String): Boolean {
        return !liveFragment!!.shouldShowRationale(deniedPermission)
    }


    fun hasPermissions(context: Context, @Size(min = 1) vararg perms: String): Boolean {
        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.M) {
            Log.w(TAG, "hasPermissions: API version < M, returning true by default")
            //为了兼容小米 ov等不遵循google权限设计规范的权限申请逻辑，小于6.0的权限逻辑
            for (perm in perms) {
                val support: IPermissionSupport? = PermissionSupport.getSupport()
                return support?.hasPermission(perm) ?: true
            }
            return true
        }
        for (perm in perms) {
            if (ContextCompat.checkSelfPermission(context, perm)
                != PackageManager.PERMISSION_GRANTED) {
                return false
            }
        }
        return true
    }
}